#include "asp-domain.hpp"

// In this file, swap and copy behaviour of asp entities' assignment operators are defined to perform deep copies.

using namespace domain;

namespace domain {
  
  void swap(TermTuple& first, TermTuple& second)
  {
    using std::swap;
    swap(first.terms, second.terms);
  }

  void swap(TermTuples& first, TermTuples& second)
  {
    using std::swap;
    swap(first.tuples, second.tuples);
  }
  void swap(UnaryTerm& first, UnaryTerm& second)
  {
    using std::swap;
    swap(first.base, second.base);
    swap(first.type, second.type);
  }
  void swap(BracketedTerm& first, BracketedTerm& second)
  {
    using std::swap;
    swap(first.base, second.base);
  }
  void swap(BinaryTerm& first, BinaryTerm& second)
  {
    using std::swap;
    swap(first.left, second.left);
    swap(first.right, second.right);
    swap(first.binopType, second.binopType);
  }
  void swap(PoolingTerm& first, PoolingTerm& second)
  {
    using std::swap;
    swap(first.terms, second.terms);
  }
  void swap(Constant& first, Constant& second)
  {
    using std::swap;
    swap(first.printable, second.printable);
  }
  void swap(Variable& first, Variable& second)
  {
    using std::swap;
    swap(first.name, second.name);
  }
  void swap(AnonymousVariable& /*first*/, AnonymousVariable& /*second*/)
  {
  }
  void swap(Function& first, Function& second)
  {
    using std::swap;
    swap(first.name, second.name);
    swap(first.tuples, second.tuples);
  }
  void swap(BodyElement& first, BodyElement& second)
  {
    using std::swap;
    swap(first.publicVariables, second.publicVariables);
  }
  void swap(NafLiteral& first, NafLiteral& second)
  {
    using std::swap;
    BodyElement* b1 = &first;
    BodyElement* b2 = &second;
    swap(*b1, *b2);
    swap(first.safetyCh, second.safetyCh);
  }
  void swap(ClassicalLiteral& first, ClassicalLiteral& second)
  {
    using std::swap;    
    NafLiteral *f = &first;
    NafLiteral *s = &second;
    swap(*f, *s);
    swap(first.name, second.name);
    swap(first.strongNegated, second.strongNegated);
    swap(first.nafNegated, second.nafNegated);
    swap(first.tuples, second.tuples);
  }
  void swap(Program& first, Program& second)
  {
    using std::swap;
    swap(first.statements, second.statements);
    swap(first.metastatements, second.metastatements);
    swap(first.query, second.query);
  }
  void swap(WeightAtLevel& first, WeightAtLevel& second)
  {
    using std::swap;
    swap(first.weight, second.weight);
    swap(first.level, second.level);
    swap(first.tuple, second.tuple);
  }
  void swap(OptimizeElement& first, OptimizeElement& second)
  {
    using std::swap;
    swap(first.weightAtLevel, second.weightAtLevel);
    swap(first.literals, second.literals);
  }
  void swap(OptimizeStatement& first, OptimizeStatement& second)
  {
    using std::swap;
    swap(first.optimizeType, second.optimizeType);
    swap(first.elements, second.elements);
  }
  void swap(Body& first, Body& second)
  {
    using std::swap;
    swap(first.elements, second.elements);
    swap(first.globalVariables, second.globalVariables);
  }
  void swap(WeakConstraint& first, WeakConstraint& second)
  {
    using std::swap;
    swap(first.body, second.body);
    swap(first.weightAtLevel, second.weightAtLevel);
  }
  void swap(Rule& first, Rule& second)
  {
    using std::swap;
    swap(first.head, second.head);
    swap(first.body, second.body);
  }
  void swap(Disjunction& first, Disjunction& second)
  {
    using std::swap;
    swap(first.literals, second.literals);
  }
  void swap(Guard& first, Guard& second)
  {
    using std::swap;
    swap(first.binop, second.binop);
    swap(first.term, second.term);
    swap(first.left, second.left);
  }
  void swap(ChoiceElement& first, ChoiceElement& second)
  {
    using std::swap;
    swap(first.literal, second.literal);
    swap(first.dependencyLiterals, second.dependencyLiterals);
  }
  void swap(Choice& first, Choice& second)
  {
    using std::swap;
    swap(first.elements, second.elements);
    swap(first.guard_l, second.guard_l);
    swap(first.guard_r, second.guard_r);
  }
  void swap(AggregateElement& first, AggregateElement& second)
  {
    using std::swap;
    swap(first.termTuple, second.termTuple);
    swap(first.dependencyLiterals, second.dependencyLiterals);
  }
  void swap(Aggregate& first, Aggregate& second)
  {
    using std::swap;
    BodyElement* b1 = &first;
    BodyElement* b2 = &second;
    swap(*b1, *b2);
    swap(first.function, second.function);
    swap(first.nafNegated, second.nafNegated);
    swap(first.elements, second.elements);
    swap(first.guard_l, second.guard_l);
    swap(first.guard_r, second.guard_r);
  }
  void swap(BuiltinAtom& first, BuiltinAtom& second)
  {
    using std::swap;
    NafLiteral *f = &first;
    NafLiteral *s = &second;
    swap(*f, *s);
    swap(first.t1, second.t1);
    swap(first.t2, second.t2);
    swap(first.binop, second.binop);
  }
  void swap(LiteralBody& first, LiteralBody& second)
  {
    using std::swap;
    swap(first.literals, second.literals);
    swap(first.globalVariables, second.globalVariables);
  }


  


  TermTuple& TermTuple::operator=(TermTuple that)
  {
    swap(*this, that);
    return *this;
  }
  TermTuples& TermTuples::operator=(TermTuples that)
  {
    swap(*this, that);
    return *this;
  }
  UnaryTerm& UnaryTerm::operator=(UnaryTerm that)
  {
    swap(*this, that);
    return *this;
  }
  BracketedTerm& BracketedTerm::operator=(BracketedTerm that)
  {
    swap(*this, that);
    return *this;
  }
  BinaryTerm& BinaryTerm::operator=(BinaryTerm that)
  {
    swap(*this, that);
    return *this;
  }
  PoolingTerm& PoolingTerm::operator=(PoolingTerm that)
  {
    swap(*this, that);
    return *this;
  }
  Constant& Constant::operator=(Constant that)
  {
    swap(*this, that);
    return *this;
  }
  Variable& Variable::operator=(Variable that)
  {
    swap(*this, that);
    return *this;
  }
  AnonymousVariable& AnonymousVariable::operator=(AnonymousVariable that)
  {
    swap(*this, that);
    return *this;
  }
  ClassicalLiteral& ClassicalLiteral::operator=(ClassicalLiteral that)
  {
    swap(*this, that);
    return *this;
  }
  Program& Program::operator=(Program that)
  {
    swap(*this, that);
    return *this;
  }
  WeightAtLevel& WeightAtLevel::operator=(WeightAtLevel that)
  {
    swap(*this, that);
    return *this;
  }
  OptimizeElement& OptimizeElement::operator=(OptimizeElement that)
  {
    swap(*this, that);
    return *this;
  }
  OptimizeStatement& OptimizeStatement::operator=(OptimizeStatement that)
  {
    swap(*this, that);
    return *this;
  }
  Body& Body::operator=(Body that)
  {
    swap(*this, that);
    return *this;
  }
  WeakConstraint& WeakConstraint::operator=(WeakConstraint that)
  {
    swap(*this, that);
    return *this;
  }
  Rule& Rule::operator=(Rule that)
  {
    swap(*this, that);
    return *this;
  }
  Disjunction& Disjunction::operator=(Disjunction that)
  {
    swap(*this, that);
    return *this;
  }
  Guard& Guard::operator=(Guard that)
  {
    swap(*this, that);
    return *this;
  }
  ChoiceElement& ChoiceElement::operator=(ChoiceElement that)
  {
    swap(*this, that);
    return *this;
  }
  Choice& Choice::operator=(Choice that)
  {
    swap(*this, that);
    return *this;
  }
  AggregateElement& AggregateElement::operator=(AggregateElement that)
  {
    swap(*this, that);
    return *this;
  }
  Aggregate& Aggregate::operator=(Aggregate that)
  {
    swap(*this, that);
    return *this;
  }
  BuiltinAtom& BuiltinAtom::operator=(BuiltinAtom that)
  {
    swap(*this, that);
    return *this;
  }
  LiteralBody& LiteralBody::operator=(LiteralBody that)
  {
    swap(*this, that);
    return *this;
  }

  
  


}
